<?php

namespace Drupal\cl_components\Component;

use Drupal\cl_components\Exception\ComponentNotFoundException;
use Drupal\cl_components\Exception\TemplateNotFoundException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Twig\Error\LoaderError;
use Twig\Loader\FilesystemLoader;
use Twig\Source;

/**
 * Lets you load templates using the component ID.
 */
class ComponentLoader extends FilesystemLoader {

  /**
   * The component discovery service.
   *
   * @var \Drupal\cl_components\Component\ComponentDiscovery
   */
  protected ComponentDiscovery $componentDiscovery;

  /**
   * Checks if Twig is in debug mode.
   *
   * @var bool
   */
  private bool $debug;

  /**
   * Additional debugging tools.
   *
   * @var bool
   */
  private bool $additionalDebug;

  /**
   * Constructs a new ComponentLoader object.
   *
   * @param \Drupal\cl_components\Component\ComponentDiscovery $component_discovery
   *   The component discovery.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param ?array $twig_config
   *   The twig configuration.
   */
  public function __construct(ComponentDiscovery $component_discovery, ConfigFactoryInterface $config_factory, ?array $twig_config) {
    parent::__construct();
    $this->componentDiscovery = $component_discovery;
    $this->additionalDebug = (bool) $config_factory->get('cl_components.settings')
      ->get('debug');
    $twig_config = $twig_config ?: [];
    $this->debug = (bool) ($twig_config['debug'] ?? FALSE);
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Twig\Error\LoaderError
   *   Thrown if a template matching $name cannot be found.
   */
  protected function findTemplate($name, $throw = TRUE) {
    [$id, $variant] = $this->parseIdAndVariant($name);
    try {
      $component = $this->componentDiscovery->find($id);
      // Validate if the variant is valid.
      if ($variant && !in_array($variant, $component->getVariants())) {
        $message = sprintf('Invalid variant "%s" for component "%s".', $variant, $id);
        throw new LoaderError($message);
      }
      $template = $component->getTemplateName($variant ?? '');
      $path = sprintf(
        '%s%s%s',
        $component->getMetadata()->getPath(),
        DIRECTORY_SEPARATOR,
        $template
      );
    }
    catch (ComponentNotFoundException | TemplateNotFoundException  $e) {
      throw new LoaderError($e->getMessage(), $e->getCode(), $e);
    }
    if ($path || !$throw) {
      return $path;
    }

    throw new LoaderError(sprintf('Unable to find template "%s" in the components registry.', $name));
  }

  /**
   * {@inheritdoc}
   */
  public function exists($name): bool {
    try {
      [$id, $variant] = $this->parseIdAndVariant($name);
      $component = $this->componentDiscovery->find($id);
      return !$variant || in_array($variant, $component->getVariants());
    }
    catch (ComponentNotFoundException | LoaderError $e) {
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\cl_components\Exception\ComponentNotFoundException
   */
  public function getSourceContext($name): Source {
    [$component_id, $variant] = $this->parseIdAndVariant($name);
    $component = $this->componentDiscovery->find($component_id);
    $source = parent::getSourceContext($name);
    $code = "{{ cl_components_additional_context('" . addcslashes($component_id, "'") . "', '" . addcslashes($variant, "'") . "')}}"
      . PHP_EOL . "{{ attach_library('" . addcslashes($component->getLibraryName(), "'") . "') }}"
      . PHP_EOL . $source->getCode();
    if ($this->debug) {
      $code = "{# start cl_component $name #}" . PHP_EOL
        . "<!-- start cl_component $name -->" . PHP_EOL
        . $code . PHP_EOL
        . "<!-- end component $name -->" . PHP_EOL
        . "{# end cl_component $name #}" . PHP_EOL;
    }
    if ($this->additionalDebug) {
      $code = "{{ attach_library('cl_components/cl_debug') }}" . PHP_EOL . $code;
    }
    return new Source($code, $source->getName(), $source->getPath());
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheKey($name): string {
    return 'cl-component::' . parent::getCacheKey($name);
  }

  /**
   * Parse ID and variant from the template key.
   *
   * @param string $name
   *   The template name as provided in the include/embed.
   *
   * @return array
   *   The component ID and variant.
   *
   * @throws \Twig\Error\LoaderError
   */
  public function parseIdAndVariant(string $name): array {
    $this->validateName($name);
    // Strip '#cl(` from the name "#cl(my-component-id, my-variant)".
    $temp = substr($name, 4);
    // Remove the last character, a ')'.
    $temp = substr($temp, 0, -1);
    // If there was a variant, it is separated by a comma.
    $parts = array_map('trim', explode(',', $temp));
    return [$parts[0], $parts[1] ?? NULL];
  }

  /**
   * Ensures that the name is valid.
   *
   * @throws \Twig\Error\LoaderError
   *   When the name is invalid.
   */
  private function validateName(string $name): void {
    if (strpos($name, '#cl(') !== 0 || $name[mb_strlen($name) - 1] !== ')') {
      $message = sprintf(
        'CL Component templates should be included using #cl(my-component-id[, my-variant]). "%s" provided.',
        $name
      );
      throw new LoaderError($message);
    }
  }

}
